/*
DL2 Monitor: ./pa/paipcp.c
Copyright (C) 2002, Rolf Jger, Ulm (Germany)

See the file README for information on usage and redistribution
of this source code file, and for a DISCLAIMER OF ALL WARRANTIES.
*/

/*----------------------------------------------------------------------------
 * Datalink Monitor (paipcp): Protocol Analysis (RFC1332)
 * PPP/IP Control Protocol (IPCP)
 *
 * Implementation Notes
 * P_*  indicate frame positions of certain fields;
 * C_*  indicate known IPCP codes;
 * CO_* indicate known configuration options
 * default contains: code, request-id
 *
 * Configuration options have been assigned by 
 *   RFC1332 (specification of IPCP), 
 *   RFC1700 (latest assigned numbers)
 *   ppp-numbers (iana online document)
 * 
 * Author: Rolf Jger,Ulm
 *----------------------------------------------------------------------------
 * YYYYMMDD  History
 * 20020427  redefinition: length instead of packet_length.
 * 20020405  fully implemented.
 * 20020330  created from scratch
 * $Id: paipcp.c,v 1.7 2002/04/29 12:59:05 dv Exp $
 *----------------------------------------------------------------------------
*/
# include <stdio.h>
# include <string.h>
# include "lib/dl2pa.h"
# include "pa/ipcp.h"

# define NSTR  ""

// IPCP frame positions:
# define P_CODE  0x00
# define P_REQUESTID  0x01
# define P_LENGTH  0x02
# define P_DATA  0x04

// known codes: 1..7 (RFC1332)
# define C_Crequest  0x01
# define C_Cack      0x02
# define C_Cnak      0x03
# define C_Creject   0x04
# define C_Trequest  0x05
# define C_Tack      0x06
# define C_Cdreject  0x07

// default indications:
static const char* codestr[] = {
  NSTR,
  "ConfRequest",
  "ConfAck",
  "ConfNak",
  "ConfReject",
  "TermRequest",
  "TermAck",
  "CodeReject",
};
# define MAXCODE C_Cdreject

// known configuration options:  RFC1332
# define CO_IPaddresses    0x01  /*deprecated*/
# define CO_IPcompression  0x02  
# define CO_IPaddress      0x03  
# define CO_MobileIPv4     0x04  /*RFC2290*/
# define CO_PrimaryDNS     0x81  /*RFC1877*/
# define CO_PrimaryNBNS    0x82  /*RFC1877*/
# define CO_SecondaryDNS   0x83  /*RFC1877*/
# define CO_SecondaryNBNS  0x84  /*RFC1877*/

struct OptionFM {const char* F; int M;};
# define MAXOPT 8
struct OptionFM config[MAXOPT+1];
# define ADDOPT(i,t) {config[i].F=IPCP_F##t;config[i].M=IPCP_M##t;}

static int init_tables = 1;
static int print_option(Octet*,int);

//-exports--------------------------------------------------------------------
int dl2_paipcp(Dl2_Frame *frame,int verbose) {
  Octet *buffer = frame->buffer;
  int length = frame->length, i = 0;
  Octet code;
  Doctet pl;

  if (init_tables) { 
    int j;
    for (j = 0; j <= MAXOPT; j++) {config[j].F = NSTR; config[j].M = 0;}
    ADDOPT(CO_IPcompression,ip_compression);
    ADDOPT(CO_IPaddress,ip_address);
    ADDOPT(5,dns1s_address);
    ADDOPT(7,dns2s_address);
    init_tables = 0;
  }

  // check for required packet length:
  if (length < P_DATA) return -1;

  //code:
  code = buffer[i];
  if (verbose & IPCP_Mcode) {
    printf("%s=%02x",IPCP_Fcode,code);
  }
  else if (verbose & IPCP_Mdefault) {
    const char *cs = (code <= MAXCODE ? codestr[code] : PA_FC_UNKNOWN);
    printf(" %s",cs);
  } i = P_REQUESTID;

  // request-id:
  if (verbose & IPCP_Mrequest_id) {
    printf(" %s=%02x",IPCP_Frequest_id,buffer[i]);
  }
  else if (verbose & IPCP_Mdefault) {
    printf("(%02x)",buffer[i]);
  } i = P_LENGTH;

  // packet-length:
  if (verbose & IPCP_Mlength) {
    printf(" %s=",IPCP_Flength);
    for (; i < P_DATA; i++) printf("%02x",buffer[i]);
  } else i = P_DATA;

  // data:
  if ((pl = str2doct(buffer+P_LENGTH)) > length) pl = length;
  if (verbose & IPCP_Mdata) {
    printf(" %s=",IPCP_Fdata);
    for (; i < pl; i++) printf("%02x",buffer[i]);
  }

  // options:
  if (verbose & IPCP_Moptions) {
    for (i = P_DATA; i < pl;) {
      Octet type = buffer[i] % 0x7c;  //hack
      if (type <= MAXOPT) {
        printf(" %s=",config[type].F);
        i += print_option(buffer+i,pl);
      }
      else {
        printf(" %s=",PA_FT_UNKNOWN);
        break;
      }
    }
  }

  else for (i = P_DATA; i < pl;) {
    Octet type = buffer[i] % 0x7c;  //hack
    if (type <= MAXOPT) {
      if (config[type].M & verbose) {
        printf(" %s=",config[type].F);
        i += print_option(buffer+i,pl);
      }
      else {i += buffer[i+1]; continue;}
    }
    else {
      printf(" %s=",PA_FT_UNKNOWN);
      break;
    }
  }
  return i;
} //dl2_paipcp


//-locals---------------------------------------------------------------------
/* print one option, return number of analyzed octets, ml on error;
 * option format: <type 1><length 1><data ?>;
 * <length> indicates length including <type> and <data>.
*/
inline int print_option(Octet *buffer,int ml) {
  Octet type = buffer[0], length = buffer[1];
  int i = ml;
  if (length >= 2) {  
    if (length > ml) length = ml;
    for (i = 2; i < length; i++) printf("%02x",buffer[i]);
  } return i;
} //print_option

/***EOM***/
