/*
DL2 Monitor: ./pa/patcp.c
Copyright (C) 2002, Rolf Jger, Ulm (Germany)

See the file README for information on usage and redistribution
of this source code file, and for a DISCLAIMER OF ALL WARRANTIES.
*/

/*----------------------------------------------------------------------------
 * Datalink Monitor (patcp): Protocol Analysis (RFC793)
 * Transmission Control Protocol
 *
 * Implementation Notes
 * default contains: source-port, destination-port, sequence-number, flags.
 * 
 * Author: Rolf Jger,Ulm
 *----------------------------------------------------------------------------
 * YYYYMMDD  History
 * 20020427  data implemented.
 * 20020330  created from scratch
 * $Id: patcp.c,v 1.4 2002/04/29 19:40:03 dv Exp $
 *----------------------------------------------------------------------------
*/
# include <stdio.h>
# include <string.h>
# include "lib/dl2pa.h"
# include "pa/portmap.h"
# include "pa/tcp.h"

// TCP frame positions:
# define P_SOURCE_PORT       0x00
# define P_DESTINATION_PORT  0x02
# define P_SEQUENCE_NO       0x04
# define P_ACKNOWLEDG_NO     0x08
# define P_DATA_OFFSET       0x0c
# define P_FLAGS             0x0d
# define P_WINDOW            0x0e
# define P_CHECKSUM          0x10
# define P_URGENT_PTR        0x12
# define P_OPTIONS           0x14
# define P_DATA              hlength
# define MIN_HLENGTH         P_OPTIONS

# define FLAG_FIN  0x01
# define FLAG_SYN  0x02
# define FLAG_RST  0x04
# define FLAG_PSH  0x08
# define FLAG_ACK  0x10
# define FLAG_URG  0x20
# define FLAG_MSK  0x3f


//-exports--------------------------------------------------------------------
int dl2_patcp(Dl2_Frame *frame,int verbose) {
  Octet *buffer = frame->buffer;
  int length = frame->length, hlength, i = P_SOURCE_PORT;

  // check for required packet length:
  if (length < MIN_HLENGTH) return -1;

  // compute header-length, data-length:
  hlength = (buffer[P_DATA_OFFSET] & 0xf0)>>2;

  //source-port:
  if (verbose & TCP_Msource_port) {
    printf(" %s=%02x%02x",TCP_Fsource_port,buffer[i],buffer[i+1]);
  }
  else if (verbose & TCP_Mdefault) {
    Doctet pn = str2doct(buffer+i);
    const char *pnstr = PA_PORTNAME(pn);
    if (pnstr != NULL) printf(" %s",pnstr);
    else printf(" %02x%02x",buffer[i],buffer[i+1]);
  } i = P_DESTINATION_PORT;

  //destination-port:
  if (verbose & TCP_Mdestination_port) {
    printf(" %s=%02x%02x",TCP_Fdestination_port,buffer[i],buffer[i+1]);
  }
  else if (verbose & TCP_Mdefault) {
    Doctet pn = str2doct(buffer+i);
    const char *pnstr = PA_PORTNAME(pn);
    if (pnstr != NULL) printf(" %s",pnstr);
    else printf(" %02x%02x",buffer[i],buffer[i+1]);
  } i = P_SEQUENCE_NO;

  //sequence-number:
  if (verbose & (TCP_Msequence_number|TCP_Mdefault)) {
    printf(" %s=%02x%02x%02x%02x",TCP_Fsequence_number,
      buffer[i],buffer[i+1],buffer[i+2],buffer[i+3]);
  } i = P_ACKNOWLEDG_NO;

  //acknowledgment-number:
  if (verbose & TCP_Macknowledgment_number) {
    printf(" %s=%02x%02x%02x%02x",TCP_Facknowledgment_number,
      buffer[i],buffer[i+1],buffer[i+2],buffer[i+3]);
  } i = P_DATA_OFFSET;

  //header-length:
  if (verbose & TCP_Mheader_length) {
    printf(" %s=%02x",TCP_Fheader_length,hlength);
  } i = P_FLAGS;

  //flags:
  if (verbose & TCP_Mflags) {
    printf(" %s=%02x",TCP_Fflags,buffer[i]);
  }
  else if (verbose & TCP_Mdefault) {
    Octet flags = buffer[i];
    char flagstr[30], *p = flagstr; *p = 0;
    if (flags & FLAG_URG) {strcat(p,"URG-"); p += 4;}
    if (flags & FLAG_ACK) {strcat(p,"ACK-"); p += 4;} 
    if (flags & FLAG_PSH) {strcat(p,"PSH-"); p += 4;}
    if (flags & FLAG_RST) {strcat(p,"RST-"); p += 4;}
    if (flags & FLAG_SYN) {strcat(p,"SYN-"); p += 4;}
    if (flags & FLAG_FIN) {strcat(p,"FIN-"); p += 4;} 
    if (p > flagstr) *(p-1) = 0; else *p = '0';
    printf(" %s", flagstr);
  } i = P_WINDOW;

  //window:
  if (verbose & TCP_Mwindow) {
    printf(" %s=%02x%02x",TCP_Fwindow,buffer[i],buffer[i+1]);
  } i = P_CHECKSUM;

  //checksum
  if (verbose & TCP_Mchecksum) {
    printf(" %s=%02x%02x",TCP_Fchecksum,buffer[i],buffer[i+1]);
  } i = P_URGENT_PTR;

  //urgent-pointer:
  if (verbose & TCP_Murgent_pointer) {
    printf(" %s=%02x%02x",TCP_Furgent_pointer,buffer[i],buffer[i+1]);
  } i = P_OPTIONS;

  //options:
  if (verbose & TCP_Moptions) {
    printf(" %s=",TCP_Foptions);
    for (; i < P_DATA; i++) printf("%02x",buffer[i]);
  } i = P_DATA;

  //data:
  if (verbose & TCP_Mdata) {
    printf(" %s=",TCP_Fdata); 
    for (;i < length; i++) printf("%02x",buffer[i]);   
  } i = length;
  return i;
} //dl2_patcp


//-locals---------------------------------------------------------------------

/***EOM***/
